import React, { Dispatch, HTMLProps } from 'react';
import { MuiThemeProvider as ThemeProvider, StylesProvider } from "@material-ui/core/styles";
import { studentTheme } from "./ui/theme";
import { MuiPickersUtilsProvider } from "@material-ui/pickers";
import MomentUtils from "@date-io/moment";
import { BrowserRouter, Link, Route, Switch } from "react-router-dom"
import moment, { Moment } from "moment";
import { route, routes } from "@/routing";
import { Provider, useDispatch, useSelector } from "react-redux";
import store, { persistor } from "@/state/store";
import { AppState } from "@/state/reducer";
import { StudentAction, StudentActions } from "@/state/actions/student";
import { sampleStudent } from "@/provider/dummy/student";
import { Trans, useTranslation } from "react-i18next";
import { StudentState } from "@/state/reducer/student";
import { Student } from "@/data";

import "moment/locale/pl"
import '@/styles/overrides.scss'
import '@/styles/header.scss'
import classNames from "classnames";
import { Button } from "@material-ui/core";
import { PersistGate } from 'redux-persist/integration/react';

moment.locale("pl")

class LocalizedMomentUtils extends MomentUtils {
    getDatePickerHeaderText(date: Moment): string {
        return this.format(date, "d MMM yyyy");
    }
}

const UserMenu = (props: HTMLProps<HTMLUListElement>) => {
    const student = useSelector<AppState, Student>(state => state.student as Student);
    const dispatch = useDispatch<Dispatch<StudentAction>>();
    const { t } = useTranslation();

    const handleUserLogin = () => {
        dispatch({
            type: StudentActions.Login,
            student: sampleStudent,
        })
    }

    const handleUserLogout = () => {
        dispatch({ type: StudentActions.Logout })
    }

    return <ul {...props}>
        {
            student ? <>
                <Trans t={ t } i18nKey="logged-in-as">logged in as <strong>{{ name: `${student.name} ${student.surname}` }}</strong></Trans>
                {' '}
                (<Link to={'#'} onClick={ handleUserLogout }>{ t('logout') }</Link>)
            </> : <>
                <Link to={'#'} onClick={ handleUserLogin }>{ t('login') }</Link>
            </>
        }
    </ul>;
}

const LanguageSwitcher = ({ className, ...props }: HTMLProps<HTMLUListElement>) => {
    const { i18n } = useTranslation();

    const handleLanguageChange = (language: string) => () => {
        i18n.changeLanguage(language);
    }

    const isActive = (language: string) => language.toLowerCase() === i18n.language.toLowerCase();

    return <ul className={ classNames(className, "language-switcher") } { ...props }>
        { ['pl', 'en'].map(language => <li key={ language }>
            <Link to="#" onClick={ handleLanguageChange(language) }
                  className={ classNames("language-switcher__language", isActive(language) && "language-switcher__language--active") }>
                { language }
            </Link>
        </li>) }
    </ul>
}

function App() {
    return (
        <Provider store={ store }>
            <PersistGate loading={ null } persistor={ persistor }>
                <StylesProvider injectFirst>
                    <MuiPickersUtilsProvider utils={ LocalizedMomentUtils } libInstance={ moment }>
                        <ThemeProvider theme={ studentTheme }>
                            <BrowserRouter>
                                <header className="header">
                                    <div id="logo" className="header__logo">
                                        <Link to={ route('home') }>
                                            <img src="img/pg-logotyp.svg"/>
                                        </Link>
                                    </div>
                                    <div className="header__nav">
                                        <nav className="header__top">
                                            <ul className="header__menu"></ul>
                                            <UserMenu className="header__user"/>
                                            <div className="header__divider" />
                                            <LanguageSwitcher className="header__language-switcher"/>
                                        </nav>
                                        <nav className="header__bottom">
                                            <ul className="header__menu header__menu--main"></ul>
                                        </nav>
                                    </div>
                                </header>
                                <Switch>{ routes.map(({ name, content, ...route }) => <Route { ...route } key={ name }>{ content() }</Route>) }</Switch>
                            </BrowserRouter>
                        </ThemeProvider>
                    </MuiPickersUtilsProvider>
                </StylesProvider>
            </PersistGate>
        </Provider>
    );
}

export default App;
